/*
 *  Copyright 2001 Adrian Thurston <adriant@ragel.ca>
 */

/*  This file is part of Ragel.
 *
 *  Ragel is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  Ragel is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with Ragel; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */


#include "fsm.h"

#include "fsmgraph.cpp"
template struct FsmGraph<State, int, Trans>;

using std::ostream;
using std::endl;

/**
 * Print a single transition. Prints target state, onChar and the integer
 * values of any functions on the transition.
 */
void dumpTransition( ostream &out, int onChar, Trans *trans )
{
	out << " " << trans->toState->num << " " << onChar << 
			" [" << trans->priority << "](";

	/* Print the functions, adding in commas in between. */
	Trans::TransFuncEl *tfel = trans->transFuncTable.table;
	int nfunc = trans->transFuncTable.tableLength;
	for ( int i = 0; i < nfunc-1; i++, tfel++ )
		out << tfel->value << ",";

	/* Print the last function without the comma. */
	if ( nfunc > 0 )
		out << tfel->value;
		
	out << ") |";
}

void dumpTransitions( ostream &out, State::TransListType *list )
{
	State::TransEl *tel = list->table;
	int ntel = list->tableLength;
	for ( int i = 0; i < ntel; i++, tel++ )
		dumpTransition( out, tel->key, tel->value );

	out << endl;
}

void dumpGraph( ostream &out, Fsm *fsm )
{
	/* Set the state numbers so we print something nice! */
	fsm->SetStateNumbers();

	/* Some header info. */
	out << "StartState: " << fsm->startState->num << " ";
	out << "Final States:";

	/* Dump Final states. */
	State **st = fsm->finStateSet.table;
	int nst = fsm->finStateSet.tableLength;
	for ( int i = 0; i < nst; i++, st++ )
		out << " " << (*st)->num << " |";
	out << endl;

	/* Walk the list of states, printing. */
	State *state = fsm->head;
	while (state) {
		out << state->num << " [" << state->outPriority << "] (";

		/* Print the functions, adding in commas in between. */
		Trans::TransFuncEl *tfel = state->outTransFuncTable.table;
		int nfunc = state->outTransFuncTable.tableLength;
		for ( int i = 0; i < nfunc-1; i++, tfel++ )
			out << tfel->value << ",";

		/* Print the last function without the comma. */
		if ( nfunc > 0 )
			out << tfel->value;

		out << ")\n->";

		dumpTransitions(out, &state->outList);
		state = state->next;
	}
}
